@@ -1,4 +1,4 @@
-/* Copyright (c) 2024-2025 LunarG, Inc.
+/* Copyright (c) 2024-2026 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -105,8 +105,11 @@ const Type& TypeManager::AddType(std::unique_ptr<Instruction> new_inst, SpvType
             break;
         case SpvType::kStruct:
             break;  // don't track structs currently
-        case SpvType::kCooperativeVectorNV:
+        case SpvType::kVectorIdEXT:
             break;  // don't track coopvec currently
+        case SpvType::kHitObjectNV:
+        case SpvType::kHitObjectEXT:
+            break;  // don't track hit objects currently
         default:
             assert(false && "unsupported SpvType");
             break;
@@ -403,7 +406,7 @@ const Type& TypeManager::GetTypePointerBuiltInInput(spv::BuiltIn built_in) {
             return GetTypePointer(spv::StorageClassInput, vec4);
         }
         default: {
-            assert(false && "unhandled builtin");
+            assert(false && "unhandled BuiltIn");
             return *(id_to_type_.begin()->second);
         }
     }
@@ -482,12 +485,16 @@ const Constant& TypeManager::AddConstant(std::unique_ptr<Instruction> new_inst,
 
     if (inst->Opcode() == spv::OpConstant) {
         if (type.inst_.Opcode() == spv::OpTypeInt && type.inst_.Word(2) == 32) {
-            int_32bit_constants_.push_back(new_constant);
-        } else if (type.inst_.Opcode() == spv::OpTypeFloat && type.inst_.Word(2) == 32) {
-            float_32bit_constants_.push_back(new_constant);
+            int_32bit_constants_.emplace_back(new_constant);
+        } else if (type.inst_.Opcode() == spv::OpTypeFloat) {
+            if (type.inst_.Word(2) == 16) {
+                float_16bit_constants_.emplace_back(new_constant);
+            } else if (type.inst_.Word(2) == 32) {
+                float_32bit_constants_.emplace_back(new_constant);
+            }
         }
     } else if (inst->Opcode() == spv::OpConstantNull) {
-        null_constants_.push_back(new_constant);
+        null_constants_.emplace_back(new_constant);
     }
 
     return *new_constant;
@@ -502,6 +509,15 @@ const Constant* TypeManager::FindConstantInt32(uint32_t type_id, uint32_t value)
     return nullptr;
 }
 
+const Constant* TypeManager::FindConstantFloat16(uint32_t type_id, uint32_t value) const {
+    for (const auto constant : float_16bit_constants_) {
+        if (constant->type_.Id() == type_id && value == constant->inst_.Word(3)) {
+            return constant;
+        }
+    }
+    return nullptr;
+}
+
 const Constant* TypeManager::FindConstantFloat32(uint32_t type_id, uint32_t value) const {
     for (const auto constant : float_32bit_constants_) {
         if (constant->type_.Id() == type_id && value == constant->inst_.Word(3)) {
@@ -561,6 +577,21 @@ const Constant& TypeManager::GetConstantOneUint32() {
     return *uint_32bit_one_constants_;
 }
 
+// It is common to use float16(0) as a default, so having it cached is helpful
+const Constant& TypeManager::GetConstantZeroFloat16() {
+    if (!float_16bit_zero_constants_) {
+        const Type& float_16_type = GetTypeFloat(16);
+        float_16bit_zero_constants_ = FindConstantFloat16(float_16_type.Id(), 0);
+        if (!float_16bit_zero_constants_) {
+            const uint32_t constant_id = module_.TakeNextId();
+            auto new_inst = std::make_unique<Instruction>(4, spv::OpConstant);
+            new_inst->Fill({float_16_type.Id(), constant_id, 0});
+            float_16bit_zero_constants_ = &AddConstant(std::move(new_inst), float_16_type);
+        }
+    }
+    return *float_16bit_zero_constants_;
+}
+
 // It is common to use float(0) as a default, so having it cached is helpful
 const Constant& TypeManager::GetConstantZeroFloat32() {
     if (!float_32bit_zero_constants_) {
